__version__ = "0.2"
__git_commiter_name__ = "Tal Yalon"
__git_commiter_email__ = "yalon@infinidat.com"
__git_branch__ = '(detached from v0.2)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = '34ecabca48b79636bbbca4350aaea1ffe22f33cc'
__git_head_subject__ = 'removed dependency on boost'
__git_head_message__ = ''
__git_dirty_diff__ = ''